/*

v 1.5 - added analog control over LED's with dimmable option  

 Forked from:
 (original components for reading serial commands from Bluetooth and extracting content)
 PROJECT: Andruino 0.1 Alpha
 PROGRAMMER: Hazim Bitar (techbitar at gmail dot com)
 DATE: July1, 2013
 FILE: Andruino.ino
 LICENSE: Public domain
*/

#include <Bounce.h>
#include <SoftwareSerial.h>
#include <EEPROM.h>
#include <MsTimer2.h>      // important note - MS2Timer2 affects PWM of PIN 3 and 11, so don't use them for PWM output.
#include <Wire.h>
#include "RTClib.h"        // real time clock
RTC_DS1307 rtc;

//EEPROM Stored Values - Addresses
int deckSchedEnabledAddr = 0;
int deckStartTimeHRAddr = 1;
int deckStartTimeMinAddr = 2;
int deckOffTimeHRAddr = 3;
int deckOffTimeMinAddr = 4;

int landSchedEnabledAddr = 5;
int landStartTimeHRAddr = 6;
int landStartTimeMinAddr = 7;
int landOffTimeHRAddr = 8;
int landOffTimeMinAddr = 9;
int landUseDuskSensorAddr = 10;
int deckUseDuskSensorAddr = 11;
int landDuskDurationAddr = 12;
int deckDuskDurationAddr = 13;
int duskLightSensorTriggerAddr = 14;
int landLEDintensityAddr = 15;
int deckLEDintensityAddr = 16;
// END EEPROM stored values

//Variables to store EEPROM-stored values
int deckSchedEnabled = 0;
int deckStartTimeHR = 0;
int deckStartTimeMin = 0;
int deckOffTimeHR = 0;
int deckOffTimeMin = 0;
int deckUseDuskSensor = 0;
int deckDuskDuration = 0;

int landSchedEnabled = 0;
int landStartTimeHR = 0;
int landStartTimeMin = 0;
int landOffTimeHR = 0;
int landOffTimeMin = 0;
int landUseDuskSensor = 0;
int landDuskDuration = 0;
//END Variables to store EEPROM-stored values

//Lighth Sensor Vars
int lightSensorValue = 0;  // variable to store the value coming from the sensor
long cumLightSensorVal = 0;
int avgLightSensorValue = 0;
double counter = 0;
//double timeElapsed = 0;
boolean duskTriggered = false;
boolean landDuskTriggered = false;
//boolean landDuskTriggeredOnceToday = false;
boolean deckDuskTriggered = false;
boolean duskTriggeredOnceToday = false;
int timeDuskTriggeredToday = 0;   //Time dusk triggered each day. Use start of day 0:00 minutes.
int duskLightSensorTriggerVal = 870;  //Default Trigger value for light sensor readings.  Higher = darker Note - value can be set using Android app and stored in EEPROM
int defaultduskLightSensorTriggerVal = 870;  //set a default value we can always go back to
//END Light Sensor Vars

// LED light Analog values to start with - eventually will be saved to EEPROM and called
int landLEDintensityVal = 255;
int deckLEDintensityVal = 255;
// END LED Light Analog Values

String inText;
int landDuskTimeStampNow;
int deckDuskTimeStampNow;

boolean landBypassed = false;
boolean deckBypassed = false;
boolean landLEDon = false;
boolean deckLEDon = false;

int delayVal = 50;  //in MS  //original was 500ms.
int statusPinIntensity = 150;  //for the analog status LEDs - set the light intensitiy

//Temp Sensor Variables
float voltsPerDegree = 0.02; // change to 0.01 for TMP35 &amp; 3
float celsius;
float farenheit;
//End Temp Sensor Variables

boolean sendLightSensReadings;

#define START_CMD_CHAR '*'
#define END_CMD_CHAR '#'
#define DIV_CMD_CHAR '|'
#define CMD_DIGITALWRITE 10
#define CMD_ANALOGWRITE 11
#define CMD_TEXT 12
#define CMD_1 1
#define CMD_2 2
#define CMD_3 3
#define CMD_4 4
#define CMD_5 5
#define CMD_6 6
#define CMD_7 7
#define CMD_POLL_STATUS 99
#define MAX_COMMAND 20  // max command number code. used for error checking.
#define MIN_COMMAND 10  // minimum command number code. used for error checking. 
#define IN_STRING_LENGHT 40
#define MAX_ANALOGWRITE 255
//#define PIN_HIGH 3
//#define PIN_LOW 2

//Bluetooth Pins
#define RxPin 0  //goes to TX pin on BT module
#define TxPin 1 //goes to RX pin on BT module
SoftwareSerial blueToothSerial(RxPin, TxPin); // RX, TX

//Arduino Inputs and Outputs
//Pins being used:
//D5 - switch pin
//D6 - switch pin
const int lightSensorAnalogPin = A1;// select the input pin for the potentiometer
const int landLightTriggerPin = 9;  //the pin that will trigger the SSR to turn on the Landscape Lights (HIGH)
const int landLightStatusPin = A2;  //the pin that will be used to light up an LED indicating the Landscape Lights are on.
const int deckLightTriggerPin = 3;  //the pin that will trigger the SSR to turn on the Deck Lights (HIGH)
const int deckLightStatusPin = A3;  //the pin that will be used to light up an LED indicating the Deck Lights are on.
const int BTdataLEDPin = A0;        //pin that will be used to show BT connected and BT data transfer
const int landLightManBypassSwitch = 5; //pin used as input from switch to manually trigger landscape lights on/off
const int deckLightManBypassSwitch = 6; //pin used as input from switch to manually trigger landscape lights on/off
// Instantiate a Bounce object with a 5 millisecond debounce time
Bounce landSWBouncer = Bounce(landLightManBypassSwitch, 5); 
Bounce deckSWBouncer = Bounce(deckLightManBypassSwitch, 5); 

//RTC - SCL goes to A5
//RTC - SDA goes to A4

const int tempSensorPin = A6;   //Temp Sensor Input 
//END Arduino Inputs and Outputs

void setup() {

  pinMode(lightSensorAnalogPin, INPUT);      
  pinMode(landLightTriggerPin, OUTPUT); 
  pinMode(landLightStatusPin, OUTPUT); 
  pinMode(deckLightTriggerPin, OUTPUT);    
  pinMode(deckLightStatusPin, OUTPUT); 
  pinMode(landLightManBypassSwitch, INPUT);      
  pinMode(deckLightManBypassSwitch, INPUT);  

  //start the lights off on system boot
  analogWrite(landLightTriggerPin, 0);
  analogWrite(deckLightTriggerPin, 0);
  
  //digitalWrite(landLightTriggerPin, LOW);
  //digitalWrite(deckLightTriggerPin, HIGH);
  

//Retreive any schedule information stored in EEPROM
if (EEPROM.read(deckSchedEnabledAddr) != 255) deckSchedEnabled = EEPROM.read(deckSchedEnabledAddr); 
if (EEPROM.read(deckStartTimeHRAddr) != 255) deckStartTimeHR = EEPROM.read(deckStartTimeHRAddr); 
if (EEPROM.read(deckStartTimeMinAddr) != 255) deckStartTimeMin = EEPROM.read(deckStartTimeMinAddr); 
if (EEPROM.read(deckOffTimeHRAddr) != 255) deckOffTimeHR = EEPROM.read(deckOffTimeHRAddr); 
if (EEPROM.read(deckOffTimeMinAddr) != 255) deckOffTimeMin = EEPROM.read(deckOffTimeMinAddr); 
if (EEPROM.read(landSchedEnabledAddr) != 255) landSchedEnabled = EEPROM.read(landSchedEnabledAddr); 
if (EEPROM.read(landStartTimeHRAddr) != 255) landStartTimeHR = EEPROM.read(landStartTimeHRAddr); 
if (EEPROM.read(landStartTimeMinAddr) != 255) landStartTimeMin = EEPROM.read(landStartTimeMinAddr); 
if (EEPROM.read(landOffTimeHRAddr) != 255) landOffTimeHR = EEPROM.read(landOffTimeHRAddr); 
if (EEPROM.read(landOffTimeMinAddr) != 255) landOffTimeMin = EEPROM.read(landOffTimeMinAddr); 
if (EEPROM.read(landUseDuskSensorAddr) != 255) landUseDuskSensor = EEPROM.read(landUseDuskSensorAddr); 
if (EEPROM.read(deckUseDuskSensorAddr) != 255) deckUseDuskSensor = EEPROM.read(deckUseDuskSensorAddr); 
if (EEPROM.read(landDuskDurationAddr) != 255) landDuskDuration = EEPROM.read(landDuskDurationAddr); 
if (EEPROM.read(deckDuskDurationAddr) != 255) deckDuskDuration = EEPROM.read(deckDuskDurationAddr); 
if (EEPROM.read(duskLightSensorTriggerAddr) != 255) duskLightSensorTriggerVal = EEPROM.read(duskLightSensorTriggerAddr); 
if (EEPROM.read(landLEDintensityAddr) != 255) landLEDintensityVal = EEPROM.read(landLEDintensityAddr); 
if (EEPROM.read(deckLEDintensityAddr) != 255) deckLEDintensityVal = EEPROM.read(deckLEDintensityAddr); 
      


//end Retreive EEPROM

#ifdef AVR
  Wire.begin();
#else
  Wire1.begin(); // Shield I2C pins connect to alt I2C bus on Arduino Due
#endif
  rtc.begin();

  if (! rtc.isrunning()) {
    Serial.println("RTC is NOT running!");
    // following line sets the RTC to the date & time this sketch was compiled
    rtc.adjust(DateTime(__DATE__, __TIME__));
  }

  Serial.begin(9600);
  //Serial.println("Landscape LED Controller - Engaged!");
  Serial.flush();

  blueToothSerial.begin(9600);
  blueToothSerial.println("Landscape LED Controller - Engaged!");
  
}

//Start main program loop
//######################################################################################################
void loop()
{
  Serial.flush();
  int ard_command = 0;
  int BTParam1 = 0;            //On Hour
  int BTParam2 = 0;            //On Minute
  int BTParam3 = 0;            //Off Hour
  int BTParam4 = 0;            //Off Minute
  boolean schedule = false;    
  boolean duskSensor = false;
  int duskDuration = 0;
  char get_char = ' ';        //read serial
  // Update the debouncer

  //Temp Sensor Variables
  int tmpSensorValue;
  float volts;
  //End Temp Sensor Variables
  
  delay(delayVal);  

  //Temp Sensor Calcs
  tmpSensorValue = analogRead(tempSensorPin);
  volts = tmpSensorValue * 5.0 / 1024.0;  // convert AD units to volts
  // convert volts to celsius
  celsius = (tmpSensorValue * 5.0 / 1024.0) / voltsPerDegree;
  // standard conversion from celsius to farenheit
  farenheit = (((tmpSensorValue * 5.0 / 1024.0) / voltsPerDegree)  * 9.0 / 5.0) + 32;
  //Serial.print(tmpSensorValue, DEC);
  //Serial.print(" A/D units   ");
  //Serial.print(volts);
  //Serial.print(" volts   ");
  //Serial.print(celsius, 1);
  //Serial.print(" degrees C   ");
  //Serial.print(farenheit,1);
  //Serial.println(" degrees F");
  //End Temp Sensor Calcs


  
  DateTime now = rtc.now();
  //~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
  //convert the current time into minutes of the day (starting at 12:00AM 00:00) - will be used to determine if the lights turn on by schedule
  int currentMinsOfDay = now.hour() * 60  + now.minute();
  
  //~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~    
  // *** DO TASKS TRIGGERED BY CUSTOM COMMAND
  //~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
  
  //Stuff that may be triggered on off by custom commands can go here.  CMD_6
  
   if (sendLightSensReadings)
   {
     BTSendLtSens();
   }
  

  
  
  //~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~    
  // *** WATCH FOR MANUAL BYPASS BUTTON CLICKS ***
  //~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
  //Allows user to click a switch at side of controller to toggle either the deck or landscape lights.  
  
   if ( landSWBouncer.update() ) {
     if ( landSWBouncer.read() == HIGH) {
        //if (digitalRead(landLightTriggerPin)== HIGH)
        //if (analogRead(landLightTriggerPin) > 0)
        if (landLEDon)
        {
          //digitalWrite(landLightTriggerPin, LOW);
          analogWrite(landLightTriggerPin, 0);
          analogWrite(landLightStatusPin, 0);
          landBypassed = false;
          landLEDon = false;
        }
        else
        {
          //digitalWrite(landLightTriggerPin, HIGH);  
          analogWrite(landLightTriggerPin, landLEDintensityVal);      
          analogWrite(landLightStatusPin, statusPinIntensity);
         landBypassed = true;
         landLEDon = true;
        }
     }
   }  
  
 
   if ( deckSWBouncer.update() ) {
     if ( deckSWBouncer.read() == HIGH) {
        //if (digitalRead(deckLightTriggerPin)== HIGH)
        //if (analogRead(deckLightTriggerPin) > 0)
        if (deckLEDon)
        {
          //digitalWrite(deckLightTriggerPin, LOW);
          analogWrite(deckLightTriggerPin, 0);
          analogWrite(deckLightStatusPin, 0);
          deckBypassed = false;
          deckLEDon = false;
        }
        else
        {
          //digitalWrite(deckLightTriggerPin, HIGH);    
          analogWrite(deckLightTriggerPin, deckLEDintensityVal);    
          analogWrite(deckLightStatusPin, statusPinIntensity);
         deckBypassed = true;
         deckLEDon = true;
        }
     }
   }  
  
  
  //~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~    
  
  
  //~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~    
  // *** DUSK TRIGGER - DATA COLLECTION ***
  //~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
  //In order to ensure that the system only triggers the lights once at dusk, we need to set some variables based on the currentMinsOfDay
  //based on 0 being 12:00 hrs and 1439 being 23:59 Hrs.  If the time dusk triggered is greater, this means the day has changed and the 
  //currentMinsOfDay has started back at 0 again. A new day and we can start watching the dusk sensor again/
  if (timeDuskTriggeredToday > currentMinsOfDay)
  {
   duskTriggeredOnceToday = false;
   timeDuskTriggeredToday = 0;
  }
  
  lightSensorValue = analogRead(lightSensorAnalogPin);
  cumLightSensorVal = cumLightSensorVal + lightSensorValue;
  
  counter +=1;
  
  //Serial.println(lightSensorValue, DEC);

  //~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~    
  // *** DUSK TRIGGER ***
  //~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
  //Determine average light reading to see if dusk is triggered.
  //want to check every 5 mins.  If the average light reading crosses the threshold, can trigger event.
  // note - the higher the reading the darker it is.  Dusk is around 1000
  //we only want to start counting to check for dusk after a resonable time during they day.  4PM to 11PM should cover Winter months in Ontario
       //if ((counter * delayVal >= 10000) && !duskTriggered && !duskTriggeredOnceToday)  //FOR TESTING
       if ((counter * delayVal >= 300000) && !duskTriggered && !duskTriggeredOnceToday)
          {
            /*
            Serial.print("  CNT = ");
            Serial.print(counter);
            Serial.print("  Dly = ");
            Serial.print(delayVal);
            Serial.print("  CNT x Dly = ");
            Serial.println(counter * delayVal);
            */
           avgLightSensorValue = cumLightSensorVal / (counter - 1);
           cumLightSensorVal = 0;
           counter = 0; 
           //timeElapsed = 0;
           //Serial.print("  5 min Avg = ");
           //Serial.print(avgLightSensorValue, DEC);
           if (avgLightSensorValue > duskLightSensorTriggerVal) 
           {
             //trigger event to turn on lights.
             duskTriggered = true;
             landDuskTriggered = true;
             deckDuskTriggered = true;
             duskTriggeredOnceToday = true;
             timeDuskTriggeredToday = currentMinsOfDay;  //capture a timestamp when the system came on. Will be used above to ensure it does not come on more than once.
             
             //Serial.print("  5 min Avg = ");
             //Serial.print(avgLightSensorValue, DEC);
             //reset everything for the next day
             avgLightSensorValue = 0;
           }
          }
            //Serial.println("");
 // }
 
  //~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~    
    
    /*
    Serial.print(now.year(), DEC);
    Serial.print('/');
    Serial.print(now.month(), DEC);
    Serial.print('/');
    Serial.print(now.day(), DEC);
    Serial.print(' ');
    Serial.print(now.hour(), DEC);
    Serial.print(':');
    Serial.print(now.minute(), DEC);
    Serial.print(':');
    Serial.print(now.second(), DEC);
    Serial.println();
    
    Serial.print(" since midnight 1/1/1970 = ");
    Serial.print(now.unixtime());
    Serial.print("s = ");
    Serial.print(now.unixtime() / 86400L);
    Serial.println("d");
     // calculate a date which is 7 days and 30 seconds into the future
    DateTime future (now.unixtime() + 7 * 86400L + 30);
 
    Serial.print(" now + 7d + 30s: ");
    Serial.print(future.year(), DEC);
    Serial.print('/');
    Serial.print(future.month(), DEC);
    Serial.print('/');
    Serial.print(future.day(), DEC);
    Serial.print(' ');
    Serial.print(future.hour(), DEC);
    Serial.print(':');
    Serial.print(future.minute(), DEC);
    Serial.print(':');
    Serial.print(future.second(), DEC);
    Serial.println();
    
    
    
    
    Serial.print("Deck stuff - ");
    Serial.print("Schedule: ");
    Serial.print(deckSchedEnabled);
    Serial.print(" StartTime: ");
    Serial.print(deckStartTimeHR);
    Serial.print(":");
    Serial.print(deckStartTimeMin);
    Serial.print(" EndTime: ");
    Serial.print(deckOffTimeHR);
    Serial.print(":");
    Serial.println(deckOffTimeMin);    

    Serial.print("Land stuff - ");
    Serial.print("Schedule: ");
    Serial.print(landSchedEnabled);
    Serial.print(" StartTime: ");
    Serial.print(landStartTimeHR);
    Serial.print(":");
    Serial.print(landStartTimeMin);
    Serial.print(" EndTime: ");
    Serial.print(landOffTimeHR);
    Serial.print(":");
    Serial.println(landOffTimeMin);  

    Serial.print("Land stuff starts at: ");
    Serial.print(landStartTimeHR * 60 + landStartTimeMin);
    Serial.print(" Ends at: ");
    Serial.println(landOffTimeHR * 60 + landOffTimeMin);

    Serial.print("Deck stuff starts at: ");
    Serial.print(deckStartTimeHR * 60 + deckStartTimeMin);
    Serial.print(" Ends at: ");
    Serial.println(deckOffTimeHR * 60 + deckOffTimeMin);

    
    delay(3000);
 
    //delay(1000);
    */



  /*
  Serial.print("LandscapeTimeON=");
  Serial.print(landStartTimeHR);
  Serial.print(":");
  Serial.print(landStartTimeMin);
  Serial.print(" Cur Min:");
  Serial.print(now.minute(), DEC);
  Serial.print("   CurrentTttMins:");
  Serial.println(currentMinsOfDay, DEC);
  delay(1000);
  */
  /*
  Serial.print("DeckSchedEnabled = ");
  Serial.print(deckSchedEnabled);
  Serial.print(" deckbypassed = ");
  Serial.print(deckBypassed);
  Serial.print(" deckUseDuskSensor = ");  
  Serial.println(deckUseDuskSensor);
  Serial.print("  DeckTimeON=");
  Serial.print(deckStartTimeHR);
  Serial.print(":");
  Serial.print(deckStartTimeMin);
  Serial.print(" Cur Min:");
  Serial.print(now.minute(), DEC);
  Serial.print("   CurrentTttMins:");
  Serial.println(currentMinsOfDay, DEC);
  delay(1000);
  */

//~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~    
// *** LANDSCAPE LIGHTS - SCHEDULE ***
//~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

// When the LEDs are deliberately turned on landBypassed = true. When they are turned back off = false
if (!landBypassed)
{
  if (landSchedEnabled && !landUseDuskSensor) // user is using schedule
  {
    //If currentMinsOfDay is between the start time in mins and end time in mins, then turn on the light
    if (currentMinsOfDay >= (landStartTimeHR * 60 + landStartTimeMin) && currentMinsOfDay <= (landOffTimeHR * 60 + landOffTimeMin))
    {
      //digitalWrite(landLightTriggerPin, HIGH);
      analogWrite(landLightTriggerPin, landLEDintensityVal);
      analogWrite(landLightStatusPin, statusPinIntensity);
      landLEDon = true;
      
    } 
    else
    {
      //digitalWrite(landLightTriggerPin, LOW);
      analogWrite(landLightTriggerPin, 0);      
      analogWrite(landLightStatusPin, 0);
      landLEDon = false;
    }
  }
   else if (landUseDuskSensor && !landSchedEnabled && duskTriggered)    //user switched over to dusk sensor
  {
    //if the land dusk sensor has triggered, turn lights on for the duration set in the db.
    //Serial.println(" Made it to step 1");
    //set the starting time to work from
    if (landDuskTriggered && duskTriggered)
    {
      //Serial.println(" Made it to step 2");
      landDuskTimeStampNow = currentMinsOfDay;
      landDuskTriggered = false;
    }

    /*
    Serial.print(" Cur Min Of Day:");
    Serial.print(currentMinsOfDay);
    Serial.print("   deckDskTimeStmp + duration in mins: ");
    Serial.println(deckDuskTimeStampNow + (0.5), DEC);
    */
    
    //now check to see time elapsed.  If the current time is less than the total time elapsed since the light was turned on, then keep it on. Else turn it off
    //if (currentMinsOfDay <= (landDuskTimeStampNow + (0.1)))  //for testing
    if (currentMinsOfDay <= (landDuskTimeStampNow + (landDuskDuration * 60)))
    {
     //digitalWrite(landLightTriggerPin, HIGH);
     analogWrite(landLightTriggerPin, landLEDintensityVal);     
     analogWrite(landLightStatusPin, statusPinIntensity);
     landLEDon = true;
     //Serial.println(" Light should be on - Step 3");
    }
    else
    {
     //digitalWrite(landLightTriggerPin, LOW);
     analogWrite(landLightTriggerPin, 0);     
     analogWrite(landLightStatusPin, 0);
     landLEDon = false;
     //once the lights have gone through an ON cycle with dusk trigger, after they are turned off, reset things so it can start over the next day
     duskTriggered = false;
     //Serial.println(" Light should be off - Step 4");
    }
  }
}
//~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~


//~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~    
// *** DECK LIGHTS - SCHEDULE ***
//~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

// When the LEDs are deliberately turned on landBypassed = true. When they are turned back off = false
if (!deckBypassed)
{
  if (deckSchedEnabled && !deckUseDuskSensor) // user is using schedule
  {
    //If currentMinsOfDay is between the start time in mins and end time in mins, then turn on the light
    if (currentMinsOfDay >= (deckStartTimeHR * 60 + deckStartTimeMin) && currentMinsOfDay <= (deckOffTimeHR * 60 + deckOffTimeMin))
    {
      //digitalWrite(deckLightTriggerPin, HIGH);
      analogWrite(deckLightTriggerPin, deckLEDintensityVal);           
      analogWrite(deckLightStatusPin, statusPinIntensity);
      deckLEDon = true;
    } 
    else
    {
      //digitalWrite(deckLightTriggerPin, LOW);
      analogWrite(deckLightTriggerPin, 0);      
      analogWrite(deckLightStatusPin, 0);
      deckLEDon = false;
    }
  }
  else if (deckUseDuskSensor && !deckSchedEnabled && duskTriggered)    //user switched over to dusk sensor
  {
    //if the land dusk sensor has triggered, turn lights on for the duration set in the db.
    //Serial.println(" Made it to step 1");
    //set the starting time to work from
    if (deckDuskTriggered && duskTriggered)
    {
      //Serial.println(" Made it to step 2");
      deckDuskTimeStampNow = currentMinsOfDay;
      deckDuskTriggered = false;
    }

    /*
    Serial.print(" Cur Min Of Day:");
    Serial.print(currentMinsOfDay);
    Serial.print("   deckDskTimeStmp + duration in mins: ");
    Serial.println(deckDuskTimeStampNow + (0.5), DEC);
    */
    
    //now check to see time elapsed.  If the current time is less than the total time elapsed since the light was turned on, then keep it on. Else turn it off
    //if (currentMinsOfDay <= (deckDuskTimeStampNow + (0.1)))  for testing
    if (currentMinsOfDay <= (deckDuskTimeStampNow + (deckDuskDuration * 60)))
    {
     //digitalWrite(deckLightTriggerPin, HIGH);
     analogWrite(deckLightTriggerPin, deckLEDintensityVal);
     analogWrite(deckLightStatusPin, statusPinIntensity);
     deckLEDon = true;
     //Serial.println(" Light should be on - Step 3");
    }
    else
    {
     //digitalWrite(deckLightTriggerPin, LOW);
     analogWrite(deckLightTriggerPin, 0);
     analogWrite(deckLightStatusPin, 0);
     deckLEDon = false;
     //once the lights have gone through an ON cycle with dusk trigger, after they are turned off, reset things so it can start over the next day
     duskTriggered = false;
     //Serial.println(" Light should be off - Step 4");
    }
  }
}
//~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~


//~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~    
// *** PROCESS COMMANDS ARRIVING OVER BLUETOOTH ***
//~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
 
  // wait for incoming data
  //if (Serial.available() < 1) return; // if serial empty, return to loop().
  if (blueToothSerial.available() < 1) return; // if serial empty, return to loop().

  // parse incoming command start flag 
  //get_char = Serial.read();
  get_char = blueToothSerial.read();
  //Serial.print(get_char);  Serial.println("");
  
  if (get_char != START_CMD_CHAR) return; // if no command start flag, return to loop().
  blinkBTLED();
  // parse incoming command type
  //ard_command = Serial.parseInt(); // read the command - parse the first item in the BT data streaming in.  Each item seperated by a |
  ard_command = blueToothSerial.parseInt(); // read the command - parse the first item in the BT data streaming in.  Each item seperated by a |
  //Serial.println(ard_command);
  // parse incoming pin# and value  
  /*
  BTParam1 = Serial.parseInt(); // read bytes past |        ON Hour
  BTParam2 = Serial.parseInt();  // read bytes past |       ON Minute
  BTParam3 = Serial.parseInt(); // read bytes past |        OFF Hour
  BTParam4 = Serial.parseInt();  // read bytes past |       OFF Minute
  schedule = Serial.parseInt();  // read bytes past |       Schedule enabled or disabled 1 / 0
  duskSensor = Serial.parseInt();  // read bytes past |     Dusk Sensor enabled or disabled 1 / 0
  duskDuration = Serial.parseInt();  // read bytes past |   Dusk Duration: Time in hours to leave lights on
  */
  
  
  //~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
  // 1) GET TEXT COMMAND FROM ANDRUINO
  if (ard_command == CMD_TEXT){   
    inText =""; //clears variable for new input   
    //while (Serial.available())  {
    while (blueToothSerial.available())  {
      //char c = Serial.read();  //gets one byte from serial buffer
      char c = blueToothSerial.read();  //gets one byte from serial buffer
      delay(5);
      if (c == END_CMD_CHAR) { // if we the complete string has been read
        // add your code here
        break;
      }              
      else {
        if (c !=  DIV_CMD_CHAR) {
          inText += c; 
          delay(5);
        }
      }
    }
  }


  //~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
  // Android requested state of lights.  Sent results back
  if (ard_command == CMD_POLL_STATUS) { 
    /*
    String send_to_android = "99|";
    if (digitalRead(landLightTriggerPin) == HIGH){send_to_android.concat("1");} else {send_to_android.concat("0");};
    send_to_android.concat("|");
    if (digitalRead(deckLightTriggerPin) == HIGH){send_to_android.concat("1");} else {send_to_android.concat("0");};
    Serial.println(send_to_android);   // Example: Sending text
    */
    sendLEDStatus(now.hour(), now.minute());
    
    return;  // Done. return to loop();
  }


  //~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
  // LAND - TURN LEDS ON OFF  - flip flop depending on current status of lights
  if (ard_command == CMD_1) { 

    //if (digitalRead(landLightTriggerPin) == HIGH)
    //if (analogRead(landLightTriggerPin) > 0)
    if (landLEDon)
      {
        //digitalWrite(landLightTriggerPin, LOW);
        analogWrite(landLightTriggerPin, 0);
        analogWrite(landLightStatusPin, 0);
        landBypassed = false;
        landLEDon = false;
      }
      else
      {
        //digitalWrite(landLightTriggerPin, HIGH);   
        analogWrite(landLightTriggerPin, landLEDintensityVal);     
        analogWrite(landLightStatusPin, statusPinIntensity);
       landBypassed = true;
       landLEDon = true;
      }
    //char send_to_android[] = "Cmd #1 Landscape Trigger Received" ;
    //Serial.println(send_to_android);   // Example: Sending text
    sendLEDStatus(now.hour(), now.minute());
    
    return;  // Done. return to loop();
  }
  
  //~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~  
  // DECK - TURN LEDS ON OFF
  if (ard_command == CMD_2) { 

    //if (digitalRead(deckLightTriggerPin) == HIGH)
    //if (analogRead(deckLightTriggerPin) > 0)    
    if (deckLEDon)
      {
        //digitalWrite(deckLightTriggerPin, LOW);
        analogWrite(deckLightTriggerPin, 0);
        analogWrite(deckLightStatusPin, 0);
        deckBypassed = false;
        deckLEDon = false;
      }
      else
      {
        //digitalWrite(deckLightTriggerPin, HIGH); 
        analogWrite(deckLightTriggerPin, deckLEDintensityVal);        
        analogWrite(deckLightStatusPin, statusPinIntensity);  
        deckBypassed = true;   
        deckLEDon = true; 
      }
    
    //char send_to_android[] = "Cmd #2 Deck Trigger Received." ;
    //Serial.println(send_to_android);   // Example: Sending text
    sendLEDStatus(now.hour(), now.minute());    
    return;  // Done. return to loop();
  }
  //~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
  // LAND PROGRAMMING SCHEDULE
  if (ard_command == CMD_3) { 
    
      BTParam1 = blueToothSerial.parseInt(); // read bytes past |        ON Hour
      BTParam2 = blueToothSerial.parseInt();  // read bytes past |       ON Minute
      BTParam3 = blueToothSerial.parseInt(); // read bytes past |        OFF Hour
      BTParam4 = blueToothSerial.parseInt();  // read bytes past |       OFF Minute
      schedule = blueToothSerial.parseInt();  // read bytes past |       Schedule enabled or disabled 1 / 0
      duskSensor = blueToothSerial.parseInt();  // read bytes past |     Land Sensor enabled or disabled 1 / 0
      duskDuration = blueToothSerial.parseInt();  // read bytes past |   Land Duration: Time in hours to leave lights on
      landLEDintensityVal = blueToothSerial.parseInt();  // read bytes past |   Landscape light intensity
      //re-map value to analog write ranges 0 - 255
      landLEDintensityVal = map(landLEDintensityVal, 0, 100, 0, 255);
      //store the new value in EEPROM in event of restart.
      EEPROM.write(landLEDintensityAddr, landLEDintensityVal); 
          
          
    
    String send_to_android = " Cmd #3: Rec'd: " ;
    send_to_android.concat("Land On:") ;    
    send_to_android.concat(BTParam1) ;
    send_to_android.concat(":") ;
    send_to_android.concat(BTParam2) ;
    send_to_android.concat(" Land Off:") ;
    send_to_android.concat(BTParam3) ;
    send_to_android.concat(":") ;
    send_to_android.concat(BTParam4) ;
    send_to_android.concat(" Sched=") ;
    send_to_android.concat(schedule) ;
    send_to_android.concat(" duskSensor=") ;
    send_to_android.concat(duskSensor) ;
    send_to_android.concat(" duskDur=") ;
    send_to_android.concat(duskDuration) ;
    send_to_android.concat(" lightVal=") ;
    send_to_android.concat(landLEDintensityVal) ;
    
    Serial.println(send_to_android);   // Example: Sending text
    
    send_to_android = "";
    
    //if schedule = 1, user wants to enable schedule.  Set the values

       EEPROM.write(landSchedEnabledAddr, schedule); 
       EEPROM.write(landStartTimeHRAddr, BTParam1); 
       EEPROM.write(landStartTimeMinAddr, BTParam2); 
       EEPROM.write(landOffTimeHRAddr, BTParam3); 
       EEPROM.write(landOffTimeMinAddr, BTParam4);
       EEPROM.write(landUseDuskSensorAddr, duskSensor);
       EEPROM.write(landDuskDurationAddr, duskDuration);
       landSchedEnabled = schedule;
       landStartTimeHR = BTParam1;
       landStartTimeMin = BTParam2;
       landOffTimeHR = BTParam3;
       landOffTimeMin = BTParam4;
       landUseDuskSensor = duskSensor;
       landDuskDuration = duskDuration;

       landBypassed = false;      

    //if the schedule is not set AND the lights are not already on. Make sure they are off.
    //Decided to turn this off - can be re-enabled if need be
    /*
    if (schedule != 1 && !landLEDon) 
    {
      //digitalWrite(landLightTriggerPin, LOW);
      analogWrite(landLightTriggerPin, 0);
      analogWrite(landLightStatusPin, 0);
      landLEDon = false;
    }
    */
    
     //apply changes for landscape lights 
    if (landLEDon) 
    {
      analogWrite(landLightTriggerPin, landLEDintensityVal);              
    }
    if (landLEDintensityVal == 0)
    {
      analogWrite(landLightStatusPin, 0);    
      landLEDon = false;
    }
    
    

    //sendLEDStatus(now.hour(), now.minute());
    return;  // Done. return to loop();
  }


  //~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
  // DECK PROGRAMMING SCHEDULE
  if (ard_command == CMD_4) { 

      BTParam1 = blueToothSerial.parseInt(); // read bytes past |        ON Hour
      BTParam2 = blueToothSerial.parseInt();  // read bytes past |       ON Minute
      BTParam3 = blueToothSerial.parseInt(); // read bytes past |        OFF Hour
      BTParam4 = blueToothSerial.parseInt();  // read bytes past |       OFF Minute
      schedule = blueToothSerial.parseInt();  // read bytes past |       Schedule enabled or disabled 1 / 0
      duskSensor = blueToothSerial.parseInt();  // read bytes past |     Dusk Sensor enabled or disabled 1 / 0
      duskDuration = blueToothSerial.parseInt();  // read bytes past |   Dusk Duration: Time in hours to leave lights on
      deckLEDintensityVal = blueToothSerial.parseInt();  // read bytes past |   Deck light intensity
      //re-map value to analog write ranges 0 - 255
      deckLEDintensityVal = map(deckLEDintensityVal, 0, 100, 0, 255);
      //store the new value in EEPROM in event of restart.
      EEPROM.write(deckLEDintensityAddr, deckLEDintensityVal); 
    
    
    
    String send_to_android = " Cmd #4 Rec'd: " ;
    send_to_android.concat("Deck On:") ;    
    send_to_android.concat(BTParam1) ;
    send_to_android.concat(":") ;
    send_to_android.concat(BTParam2) ;
    send_to_android.concat(" Deck Off:") ;
    send_to_android.concat(BTParam3) ;
    send_to_android.concat(":") ;
    send_to_android.concat(BTParam4) ;
    send_to_android.concat(" Sched=") ;
    send_to_android.concat(schedule) ;  
    send_to_android.concat(" duskSensor=") ;
    send_to_android.concat(duskSensor) ;
    send_to_android.concat(" duskDur=") ;
    send_to_android.concat(duskDuration) ;
    send_to_android.concat(" lightVal=") ;
    send_to_android.concat(deckLEDintensityVal) ;
    
    Serial.println(send_to_android);   // Example: Sending text
    
    send_to_android = "";
    
  //if schedule = 1, user wants to enable schedule.  Set the values
       deckBypassed = false; 
       EEPROM.write(deckSchedEnabledAddr, schedule); 
       EEPROM.write(deckStartTimeHRAddr, BTParam1); 
       EEPROM.write(deckStartTimeMinAddr, BTParam2); 
       EEPROM.write(deckOffTimeHRAddr, BTParam3); 
       EEPROM.write(deckOffTimeMinAddr, BTParam4);
       EEPROM.write(deckUseDuskSensorAddr, duskSensor);
       EEPROM.write(deckDuskDurationAddr, duskDuration);
       deckSchedEnabled = schedule;
       deckStartTimeHR = BTParam1;
       deckStartTimeMin = BTParam2;
       deckOffTimeHR = BTParam3;
       deckOffTimeMin = BTParam4;
       deckUseDuskSensor = duskSensor;
       deckDuskDuration = duskDuration;       

    //if the schedule is not set AND the lights are not already on. Make sure they are off.
    //Decided to turn this off - can be re-enabled if need be
    /*
    if (schedule != 1 && !deckLEDon) 
    {
      analogWrite(deckLightTriggerPin, 0);  
      analogWrite(deckLightStatusPin, 0);    
      deckLEDon = false;  
    }
    */
                 
     
   //apply changes for deck lights           
   if (deckLEDon) 
   { 
     analogWrite(deckLightTriggerPin, deckLEDintensityVal);
   }
   if (deckLEDintensityVal == 0)
   {
     analogWrite(deckLightStatusPin, 0);    
     deckLEDon = false;
   }     
      
      
    //sendLEDStatus(now.hour(), now.minute());
    return;  // Done. return to loop();
  }



//~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~  
  // Adjust the controller's clock
  if (ard_command == CMD_5) { 
  
      BTParam1 = blueToothSerial.parseInt(); // read bytes past |        Clock Set Hour
      BTParam2 = blueToothSerial.parseInt();  // read bytes past |       Clock Set Minute   
  
    
    /*
    Serial.print("TimeRec On: ");
    Serial.print(BTParam1);
    Serial.print("  BTParam3: ");
    Serial.print(BTParam3);
    Serial.print("  Now Stuff: ");
    Serial.print(now.hour());
    Serial.print(" ");
    Serial.print(now.minute());
    */
    
    if ((BTParam1 < 25 & BTParam1 >= 0) && (BTParam2 < 59 && BTParam2 >= 0))
    {
      //RTC.adjust(DateTime(now.year(), now.month(), now.day(), now.hour(), now.minute() + 1, 0));
      rtc.adjust(DateTime(now.year(), now.month(), now.day(), BTParam1, BTParam2, 0));
    }
    //need to get the date again as we just updated it.
    DateTime now = rtc.now();
    //send the updated date back to the remote device
    sendLEDStatus(now.hour(), now.minute());
    return;  // Done. return to loop();
  }






//~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~  
// Process Custom Commands sent as 6|
// Commands come in the format 6|BTParam1|BTParam2|BTParam3
 
 
  if (ard_command == CMD_6) { 
      BTParam1 = blueToothSerial.parseInt(); // read bytes past |        first parameter sent
      BTParam2 = blueToothSerial.parseInt();  // read bytes past |       second parameter sent   
      BTParam3 = blueToothSerial.parseInt();  // read bytes past |       third parameter sent   

     //send back the commands received
     String send_to_android = "";
     send_to_android.concat(ard_command);
     send_to_android.concat("|");
     send_to_android.concat(BTParam1);
     send_to_android.concat("|");
     send_to_android.concat(BTParam2);
     send_to_android.concat("|");     
     send_to_android.concat(BTParam3);
     send_to_android.concat("|");     
     send_to_android.concat(sendLightSensReadings); 
     Serial.println(send_to_android);   
     

      switch (BTParam1) {
        case 1:
          //process a request to send light sensor readings every x MS.  If BTParam3 = 1 then turn it on, if BTParam3 = 0, then turn it off.
          if (BTParam2 == 1)
          {
            sendLightSensReadings = true;
          }
          else if (BTParam2 == 0)
          {
            sendLightSensReadings = false;
          }
          break;
        case 2:
          //Process various requests to set Dusk Sensor Trigger Value.
          if (BTParam2 == 1)
          {
            //set the value for the next time the app starts.
            EEPROM.write(duskLightSensorTriggerAddr, analogRead(lightSensorAnalogPin)); 
            //also update the value currently being used to the new value.
            duskLightSensorTriggerVal = analogRead(lightSensorAnalogPin);
            
          }
          else if (BTParam2 == 2)
          {
            //reset to defaults.
            EEPROM.write(duskLightSensorTriggerAddr, 255);  //clear the EEPROM.
            duskLightSensorTriggerVal = defaultduskLightSensorTriggerVal;
          }

          else if (BTParam2 == 3)
          {
            //set to defined value
            if (BTParam3 >= 0 && BTParam3 < 1024)
            {
              EEPROM.write(duskLightSensorTriggerAddr, BTParam3);  //clear the EEPROM.
              duskLightSensorTriggerVal = BTParam3;  
            }
          }
          
          
          
          break;
          
          
        default: 
          break;
          // if nothing else matches, do the default
          // default is optional
        }    
    
    return;  // Done. return to loop();
  }
  
 
 
 
 //was 500 ms but seemed slow.  Need to find a way to increase this after first poll.
  //delay(delayVal);
}
//end void loop()
//######################################################################################################


void  BTSendLtSens()
{
     String send_to_android = "Lt Sensor: ";
     send_to_android.concat(analogRead(lightSensorAnalogPin));
     Serial.println(send_to_android);   
     delay(1000);
}


void sendLEDStatus(int hrNow, int minuteNow)
{
    String send_to_android = "99|";
    if (landLEDon){send_to_android.concat("1");} else {send_to_android.concat("0");};
    //if (analogRead(landLightTriggerPin) > 0){send_to_android.concat("1");} else {send_to_android.concat("0");};
    send_to_android.concat("|");
    if (deckLEDon){send_to_android.concat("1");} else {send_to_android.concat("0");};
    //if (analogRead(deckLightTriggerPin) > 0){send_to_android.concat("1");} else {send_to_android.concat("0");};
    send_to_android.concat("|");
    send_to_android.concat(hrNow);
    send_to_android.concat(":");
   //have to deal with time as INT and minutes coming in under 10.  ie. 5:5 should be 5:05
   if (minuteNow < 10)
   {
     send_to_android.concat("0");   
   }
   send_to_android.concat(minuteNow);
   //Add Temp in C
    send_to_android.concat("|");
    //int tempC = (celsius - (int)celsius) * 100;
    //send_to_android.concat(tempC);
    send_to_android.concat(int(celsius));     //get the int part of the temp
    send_to_android.concat(".");             //do the period
    unsigned int frac;
    if(celsius >= 0)
        frac = (celsius - int(celsius)) * 10;
    else
        frac = (int(celsius)- celsius ) * 10;
    send_to_android.concat(frac);               //get the decimal part.  Note the value 10 above will return 1 decimal place.  Use 100 to return 2.
   //END Temp in C
   frac = 0;
   //Add Temp in F
    send_to_android.concat("|");
    //int tempF = (farenheit - (int)farenheit) * 100;
    //send_to_android.concat(tempF);
    send_to_android.concat(int(farenheit));       //get the int part of the temp
    send_to_android.concat(".");                 //do the period
    if(farenheit >= 0)
        frac = (farenheit - int(farenheit)) * 10;
    else
        frac = (int(farenheit)- farenheit ) * 10;
    send_to_android.concat(frac);               //get the decimal part.  Note the value 10 above will return 1 decimal place.  Use 100 to return 2.
   //END Temp F
   //Add current light sensor reading
   send_to_android.concat("|Sensor=");
   send_to_android.concat(analogRead(lightSensorAnalogPin));
   send_to_android.concat("|TrigVal=");
   send_to_android.concat(duskLightSensorTriggerVal);
   //End Add current light sensor reading
   
   Serial.println(send_to_android);   
   //blueToothSerial.println(send_to_android);   
 
 /*
    Serial.print (int(val));  //prints the int part
    Serial.print("."); // print the decimal point
    unsigned int frac;
    if(val >= 0)
        frac = (val - int(val)) * precision;
    else
        frac = (int(val)- val ) * precision;
    Serial.println(frac,DEC) ;
 */

}


//######################################################################################################
// 2a) select the requested pin# for DigitalWrite action
void set_digitalwrite(int BTParam1, int BTParam2)
{
  switch (BTParam1) {
  case 13:
    pinMode(13, OUTPUT);
    digitalWrite(13, BTParam2);  
    // add your code here      
    break;
  case 12:
    pinMode(12, OUTPUT);
    digitalWrite(12, BTParam2);   
    // add your code here       
    break;
  case 11:
    pinMode(11, OUTPUT);
    digitalWrite(11, BTParam2);         
    // add your code here 
    break;
  case 10:
    pinMode(10, OUTPUT);
    digitalWrite(10, BTParam2);         
    // add your code here 
    break;
  case 9:
    pinMode(9, OUTPUT);
    digitalWrite(9, BTParam2);         
    // add your code here 
    break;
  case 8:
    pinMode(8, OUTPUT);
    digitalWrite(8, BTParam2);         
    // add your code here 
    break;
  case 7:
    pinMode(7, OUTPUT);
    digitalWrite(7, BTParam2);         
    // add your code here 
    break;
  case 6:
    pinMode(6, OUTPUT);
    digitalWrite(6, BTParam2);         
    // add your code here 
    break;
  case 5:
    pinMode(5, OUTPUT);
    digitalWrite(5, BTParam2); 
    // add your code here       
    break;
  case 4:
    pinMode(4, OUTPUT);
    digitalWrite(4, BTParam2);         
    // add your code here 
    break;
  case 3:
    pinMode(3, OUTPUT);
    digitalWrite(3, BTParam2);         
    // add your code here 
    break;
  case 2:
    pinMode(2, OUTPUT);
    digitalWrite(2, BTParam2); 
    // add your code here       
    break;      
    // default: 
    // if nothing else matches, do the default
    // default is optional
  } 
}


void blinkBTLED()
{
  for (int i=0; i <= 5; i++){
      analogWrite(BTdataLEDPin, statusPinIntensity);
      delay(30);
      analogWrite(BTdataLEDPin, 0);
      delay(30);
   }
}
